@@ -10,11 +10,11 @@ Interchange6::Schema - Database Schema for Interchange 6
 
 =head1 VERSION
 
-0.070
+0.080
 
 =cut
 
-our $VERSION = '0.070';
+our $VERSION = '0.080';
 
 =head1 DESCRIPTION
 
@@ -39,7 +39,7 @@ use warnings;
 use base 'DBIx::Class::Schema';
 
 __PACKAGE__->load_components( 'Helper::Schema::DateTime',
-    'Helper::Schema::QuoteNames' );
+    'Helper::Schema::QuoteNames', 'Schema::Config' );
 
 __PACKAGE__->load_namespaces(
     default_resultset_class => 'ResultSet',
