@@ -25,9 +25,15 @@ Returns array reference containing one hash reference per role ready to use with
 
 =item * admin
 
+Shop administrator with full permissions.
+
+=item * user
+
+All non-anonymous users have this role.
+
 =item * anonymous
 
-=item * authenticated
+Anonymous users.
 
 =back
 
@@ -41,15 +47,15 @@ sub records {
             description => "Shop administrator with full permissions",
         },
         {
-            name        => "anonymous",
-            label       => "Anonymous",
-            description => "Customer who is not logged in",
+            name        => "user",
+            label       => "User",
+            description => "All users have this role",
         },
         {
-            name        => "authenticated",
-            label       => "Authenticated",
-            description => "Customer who is logged in",
-        },
+            name        => "anonymous",
+            label       => "Anonymous",
+            description => "Anonymous users",
+        }
     ];
 }
 
