@@ -13,6 +13,9 @@ use base 'Interchange6::Schema::Base::Attribute';
 use Interchange6::Schema::Candy -components =>
   [qw(Tree::AdjacencyList InflateColumn::DateTime TimeStamp)];
 
+use Encode;
+use Try::Tiny;
+
 =head1 DESCRIPTION
 
 Navigation is where all navigation, category and static page details are stored.  In addition
@@ -46,109 +49,98 @@ This ensures each record has a unique value and also allows for proper ordering.
 
 =head2 navigation_id
 
-  data_type: 'integer'
-  is_auto_increment: 1
-  is_nullable: 0
-  sequence: 'navigation_navigation_id_seq'
-  primary key
+Primary key.
 
 =cut
 
 primary_column navigation_id => {
     data_type         => "integer",
     is_auto_increment => 1,
-    is_nullable       => 0,
     sequence          => "navigation_navigation_id_seq",
 };
 
 =head2 uri
 
-  data_type: 'varchar'
-  default_value: (empty string)
-  is_nullable: 0
-  size: 255
-  unique constraint
+URI.
+
+Unique constraint.
+
+See L</generate_uri> method for details of how L</uri> can be created
+automatically based on the value of L</name>.
 
 =cut
 
 unique_column uri => {
     data_type     => "varchar",
     default_value => "",
-    is_nullable   => 0,
     size          => 255
 };
 
 =head2 type
 
-  data_type: 'varchar'
-  default_value: (empty string)
-  is_nullable: 0
-  size: 32
+Type, e.g.: nav, category.
 
 =cut
 
 column type =>
-  { data_type => "varchar", default_value => "", is_nullable => 0, size => 32 };
+  { data_type => "varchar", default_value => "", size => 32 };
 
 =head2 scope
 
-  data_type: 'varchar'
-  default_value: (empty string)
-  is_nullable: 0
-  size: 32
+Scope, e.g.: menu-main, top-login.
 
 =cut
 
 column scope =>
-  { data_type => "varchar", default_value => "", is_nullable => 0, size => 32 };
+  { data_type => "varchar", default_value => "", size => 32 };
 
 =head2 name
 
-  data_type: 'varchar'
-  default_value: (empty string)
-  is_nullable: 0
-  size: 255
+Name, e.g.: Hand Tools, Fly Fishing Rods.
+
+Defaults to empty string.
 
 =cut
 
 column name => {
     data_type     => "varchar",
     default_value => "",
-    is_nullable   => 0,
     size          => 255
 };
 
 =head2 description
 
-  data_type: 'varchar'
-  default_value: (empty string)
-  is_nullable: 0
-  size: 1024
+Description, e.g.: All of our hand tools, Our collection of top fly fishing rods.
+
+Defaults to empty string.
 
 =cut
 
 column description => {
     data_type     => "varchar",
     default_value => "",
-    is_nullable   => 0,
     size          => 1024
 };
 
 =head2 alias
 
-  data_type: 'integer'
-  default_value: 0
-  is_nullable: 0
+FK on L<Interchange6::Schema::Result::Navigation/navigation_id>.
+
+Can be used for things such as menus in different languages which link back
+to the primary navigation menu.
+
+Is nullable.
 
 =cut
 
 column alias =>
-  { data_type => "integer", default_value => 0, is_nullable => 0 };
+  { data_type => "integer", is_foreign_key => 1, is_nullable => 1 };
 
 =head2 parent_id
 
-  data_type: 'integer'
-  is_nullable: 1
+Used by L<DBIx::Class::Tree::AdjacencyList> to setup parent/child relationships.
+
+Is nullable.
 
 =cut
 
@@ -156,43 +148,40 @@ column parent_id => { data_type => "integer", is_nullable => 1 };
 
 =head2 priority
 
-  data_type: 'integer'
-  default_value: 0
-  is_nullable: 0
+Signed integer priority. We normally order descending.
+
+Defaults to 0.
 
 =cut
 
 column priority =>
-  { data_type => "integer", default_value => 0, is_nullable => 0 };
+  { data_type => "integer", default_value => 0 };
 
 =head2 product_count
 
-  data_type: 'integer'
-  default_value: 0
-  is_nullable: 0
+Can be used to cache product counts.
+
+Default to 0.
 
 =cut
 
 column product_count =>
-  { data_type => "integer", default_value => 0, is_nullable => 0 };
+  { data_type => "integer", default_value => 0 };
 
 =head2 created
 
-  data_type: 'datetime'
-  set_on_create: 1
-  is_nullable: 0
+Date and time when this record was created returned as L<DateTime> object.
+Value is auto-set on insert.
 
 =cut
 
 column created =>
-  { data_type => "datetime", set_on_create => 1, is_nullable => 0 };
+  { data_type => "datetime", set_on_create => 1 };
 
 =head2 last_modified
 
-  data_type: 'datetime'
-  set_on_create: 1
-  set_on_update: 1
-  is_nullable: 0
+Date and time when this record was last modified returned as L<DateTime> object.
+Value is auto-set on insert and update.
 
 =cut
 
@@ -200,24 +189,116 @@ column last_modified => {
     data_type     => "datetime",
     set_on_create => 1,
     set_on_update => 1,
-    is_nullable   => 0
 };
 
 =head2 active
 
-  data_type: 'boolean'
-  default_value: 1
-  is_nullable: 0
+Whether navigation is active and therefore should be displayed.
+
+Boolean defaults to true (1).
 
 =cut
 
 column active =>
-  { data_type => "boolean", default_value => 1, is_nullable => 0 };
+  { data_type => "boolean", default_value => 1 };
 
 =head1 METHODS
 
 Attribute methods are provided by the L<Interchange6::Schema::Base::Attribute> class.
 
+=head2 insert
+
+Override inherited method to call L</generate_uri> method in case L</name>
+has been supplied as an argument but L</uri> has not.
+
+=cut
+
+sub insert {
+    my ( $self, @args ) = @_;
+    if ( $self->name && !$self->uri ) {
+        $self->generate_uri;
+    }
+    $self->next::method(@args);
+    return $self;
+}
+
+=head2 generate_uri($attrs)
+
+Called by L</new> if no uri is given as an argument.
+
+The following steps are taken:
+
+=over
+
+1. Stash C<< $self->name >> in C<$uri> to allow manipulation via filters
+
+2. Remove leading and trailing spaces and replace remaining spaces and
+C</> with C<->
+
+3. Search for all rows in L<Interchange6::Schema::Result::Setting> where
+C<scope> is C<Navigation> and C<name> is <generate_uri_filter>
+
+4. For each row found eval C<< $row->value >>
+
+5. Finally set the value of column L</uri> to C<$uri>
+
+=back
+
+Filters stored in L<Interchange6::Schema::Result::Setting> are executed via
+eval and have access to C<$uri> and also the navigation result held in 
+C<$self>
+
+Examples of filters stored in Setting might be:
+
+    {
+        scope => 'Navigation',
+        name  => 'generate_uri_filter',
+        value => '$uri =~ s/badstuff/goodstuff/gi',
+    },
+    {
+        scope => 'Navigation',
+        name  => 'generate_uri_filter',
+        value => '$uri = lc($uri)',
+    },
+
+=cut
+
+sub generate_uri {
+    my $self = shift;
+
+    my $uri = $self->name;
+
+    # make sure we have clean utf8
+    try {
+        $uri = Encode::decode( 'UTF-8', $uri, Encode::FB_CROAK )
+          unless utf8::is_utf8($uri);
+    }
+    catch {
+        $self->throw_exception(
+            "Navigation->generate_uri failed to decode UTF-8 text: $_" );
+    };
+
+    $uri =~ s/^\s+//;       # remove leading space
+    $uri =~ s/\s+$//;       # remove trailing space
+    $uri =~ s{[\s/]+}{-}g;  # change space and / to -
+
+    my $filters = $self->result_source->schema->resultset('Setting')->search(
+        {
+            scope => 'Navigation',
+            name  => 'generate_uri_filter',
+        },
+    );
+
+    while ( my $filter = $filters->next ) {
+        eval $filter->value;
+        $self->throw_exception("Navigation->generate_uri filter croaked: $@")
+          if $@;
+    }
+
+    $self->uri($uri);
+}
+
+
 =head2 siblings_with_self
 
 Similar to the inherited L<siblings|DBIx::Class::Tree::AdjacencyList/siblings> method but also returns the object itself in the result set/list.
@@ -251,7 +332,7 @@ L<ancestors|DBIx::Class::Tree::AdjacencyList/ancestors>
 
 =item *
 
-L<as_descendant|DBIx::Class::Tree::AdjacencyList/as_descendant>
+L<has_descendant|DBIx::Class::Tree::AdjacencyList/has_descendant>
 
 =item *
 
@@ -295,6 +376,26 @@ __PACKAGE__->parent_column('parent_id');
 
 =head1 RELATIONS
 
+=head2 active_children
+
+Related object: L<Interchange6::Schema::Result::Navigation>
+
+Conditions: self.parent_id = foreign.navigation_id && foreign.active = 1
+
+=cut
+
+has_many
+  active_children => "Interchange6::Schema::Result::Navigation",
+  sub {
+    my $args = shift;
+
+    return {
+        "$args->{foreign_alias}.parent_id" =>
+          { -ident => "$args->{self_alias}.navigation_id" },
+        "$args->{foreign_alias}.active" => 1,
+    };
+  };
+
 =head2 navigation_products
 
 Type: has_many
@@ -333,4 +434,16 @@ has_many
   "navigation_id",
   { cascade_copy => 0, cascade_delete => 0 };
 
+=head2 attributes
+
+Type: many_to_many
+
+Accessor to related attribute results.
+
+=cut
+
+many_to_many
+  attributes => "navigation_attributes", "attribute";
+
+
 1;
